'===============================================================================
'                  Language Tools for PSPad
'                 
' This script works only in the PSPad editor, see http://www.pspad.com.
' It transforms lists of words in their basic form into inflected forms.
' As it stands, it only works for German, but it might be useful for
' other languages if modified accordingly.
' Made with and for PSPad by Jan Schreiber, a.k.a. MrSpock, January 2007.
'
'===============================================================================

Option explicit
const module_name  = "Language Tools" ' Modify these
const module_ver   = "2.0"            ' to suit
const menu_item1   = "Deklinieren"    ' your needs
const menu_item2   = "Zeit"
const menu_item3   = "Tageszeiten"
const menu_item4   = "Konjugieren"
const menu_item5   = "Pre- and Suffix"
const menu_item6   = "Edit Script"
const confText     = "Do you want to apply this to the whole document?"
const confTitle    = "Confirmation"
const showStats    = True
const openInNew    = True
Dim sel, i, j, k, myEditor, inputText, outputText(), sizeOfSuffix, sizeOfInput, sizeOfOutput

'===============================================================================
'                  Add menu items
'===============================================================================
Sub Init
  addMenuItem menu_item1, module_name, "Deklinieren", "Ctrl+Shift+d"
  addMenuItem menu_item2, module_name, "Zeit"
  addMenuItem menu_item3, module_name, "Tageszeiten"
  addMenuItem menu_item4, module_name, "Konjugieren"
  addMenuItem menu_item5, module_name, "PreSuff"
  addMenuItem "-", module_name, ""
  addMenuItem menu_item6, module_name, "OpenScript"
End Sub

'===============================================================================
'                  Open this script file in the editor
'===============================================================================
Sub OpenScript
  set myEditor = newEditor()
  myEditor.openFile(moduleFileName(module_name))
End Sub

'===============================================================================
'                  Deklinieren
'===============================================================================
Sub Deklinieren
  Dim suffix(5) ' An array of strings containing the inflection suffixes
  suffix(0) = ""
  suffix(1) = "e"
  suffix(2) = "em"
  suffix(3) = "en"
  suffix(4) = "er"
  suffix(5) = "es"
  sizeOfSuffix = UBound(suffix,1)    ' get length of suffix array

  If editorsCount = 0 Then Exit Sub
  Set myEditor = newEditor           ' new editor object
  myEditor.assignActiveEditor        ' assign the active window to the object

  If myEditor.selText="" Then        ' In case selection is empty ...
    If MsgBox(confText, 36, confTitle) = vbNo Then Exit Sub ' ask if user wants to quit, if not ...
    inputText = split(myEditor.Text,vbCrLf,-1) ' get all text from active window, parse into array.
    sel = 0
  Else
    inputText = split(myEditor.selText,vbCrLf,-1) ' Get selected text, parse into array
    sel = 1
  End If
  sizeOfInput = UBound(inputText,1)  ' Get length of input array (number of lines)
  sizeOfOutput = (sizeOfInput+1) * (sizeOfSuffix+1) - 1 ' Calculate length for output array and ...
  ReDim outputText(sizeOfOutput)                        ' resize the latter accordingly

  i = 0 ' i is a counter for array element in output array
  For j = 0 To sizeOfInput Step 1
    For k = 0 To sizeOfSuffix Step 1
      If Not inputText(j) = "" Then
         outputText(i) = RTrim(inputText(j)) & suffix(k) ' RTrim deletes trailing spaces
      End If
      i = i + 1
      If i > sizeOfOutput Then Exit For
    Next
  Next

  If sel = 0 Then
    myEditor.Text(Join(outputText,vbCrLF))
  Else
    myEditor.selText(Join(outputText,vbCrLF))
  End If

End Sub

'===============================================================================
'                  Zahlwrter (numerals)
'===============================================================================
Sub Zeit
  Dim suffix(9) ' An array of strings containing the suffixes
  suffix(0) = "jhrig"
  suffix(1) = "monatig"
  suffix(2) = "wchig"
  suffix(3) = "stndig"
  suffix(4) = "mintig"
  suffix(5) = "malig"
  suffix(6) = "fach"
  suffix(7) = "stckig"
  suffix(8) = "silbig"
  suffix(9) = "semestrig"
  sizeOfSuffix = UBound(suffix,1)    ' get length of suffix array

  If editorsCount = 0 Then Exit Sub
  Set myEditor = newEditor           ' new editor object
  myEditor.assignActiveEditor        ' assign the active window to the object
  If myEditor.selText="" Then        ' If no text is selected, then ...
    If MsgBox(confText, 36, confTitle) = vbNo Then Exit Sub
    inputText = split(myEditor.Text,vbCrLf,-1) ' get all text from active window, parse into array
    sel = 0
  Else
    inputText = split(myEditor.selText,vbCrLf,-1) ' Get selected text, parse into array
    sel = 1
  End If
  sizeOfInput = UBound(inputText,1)  ' Get length of input array (number of lines)
  sizeOfOutput = (sizeOfInput+1) * (sizeOfSuffix+1) - 1 ' Calculate length for output array and ...
  ReDim outputText(sizeOfOutput)                        ' resize the latter accordingly

  i = 0 ' counter for array element in output array
  For j = 0 To sizeOfInput Step 1
    For k = 0 To sizeOfSuffix Step 1
      If Not inputText(j) = "" Then
         outputText(i) = RTrim(inputText(j)) & suffix(k) ' RTrim deletes trailing spaces
      End If
      i = i + 1
      If i > sizeOfOutput Then Exit For
    Next
  Next

  If sel = 0 Then
    myEditor.Text(Join(outputText,vbCrLF))
  Else
    myEditor.selText(Join(outputText,vbCrLF))
  End If

End Sub

'===============================================================================
'                  Tageszeiten
'===============================================================================
Sub Tageszeiten
  Dim suffix(7) ' An array of strings containing the suffixes
  suffix(0) = "nachmittag"
  suffix(1) = "vormittag"
  suffix(2) = "morgen"
  suffix(3) = "abend"
  suffix(4) = "nacht"
  suffix(5) = "mittag"
  suffix(6) = "abends"
  suffix(7) = "s"
  sizeOfSuffix = UBound(suffix,1)    ' get length of suffix array

  If editorsCount = 0 Then Exit Sub
  Set myEditor = newEditor           ' new editor object
  myEditor.assignActiveEditor        ' assign the active window to the object
  If myEditor.selText="" Then
    If MsgBox(confText, 36, confTitle) = vbNo Then Exit Sub
    inputText = split(myEditor.Text,vbCrLf,-1) ' Get all text from active window, parse into array
    sel = 0
  Else
    inputText = split(myEditor.selText,vbCrLf,-1) ' Get selected text, parse into array
    sel = 1
  End If
  sizeOfInput = UBound(inputText,1)  ' Get length of input array (number of lines)
  sizeOfOutput = (sizeOfInput+1) * (sizeOfSuffix+1) - 1 ' Calculate length for output array and ...
  ReDim outputText(sizeOfOutput)                        ' resize the latter accordingly

  i = 0 ' counter for array element in output array
  For j = 0 To sizeOfInput Step 1
    For k = 0 To sizeOfSuffix Step 1
      If Not inputText(j) = "" Then
         outputText(i) = RTrim(inputText(j)) & suffix(k) ' RTrim deletes trailing spaces
      End If
      i = i + 1
      If i > sizeOfOutput Then Exit For
    Next
  Next

  If sel = 0 Then
    myEditor.Text(Join(outputText,vbCrLF))
  Else
    myEditor.selText(Join(outputText,vbCrLF))
  End If

End Sub

'===============================================================================
'                  Pre- and Suffix
'===============================================================================
Sub PreSuff

  Dim sizeOfPrefix
  Dim prefix(3) ' An array of strings containing the prefixes
  prefix(0) = "test1"
  prefix(1) = "test2"
  prefix(2) = "test3"
  prefix(3) = "test4"
  sizeOfPrefix = UBound(prefix,1)    ' get length of prefix array

  Dim suffix(3) ' An array of strings containing the suffixes
  suffix(0) = "suff1"
  suffix(1) = "suff2"
  suffix(2) = "suff3"
  suffix(3) = "suff4"
  sizeOfSuffix = UBound(suffix,1)    ' get length of suffix array

  If editorsCount = 0 Then Exit Sub
  Set myEditor = newEditor           ' new editor object
  myEditor.assignActiveEditor        ' assign the active window to the object
  If myEditor.selText="" Then
    If MsgBox(confText, 36, confTitle) = vbNo Then Exit Sub
    inputText = split(myEditor.Text,vbCrLf,-1) ' Get all text from active window, parse into array
    sel = 0
  Else
    inputText = split(myEditor.selText,vbCrLf,-1) ' Get selected text, parse into array
    sel = 1
  End If
  sizeOfInput = UBound(inputText,1)  ' Get length of input array (number of lines)
  sizeOfOutput = (sizeOfInput+1) * (sizeOfSuffix+1) * (sizeOfPrefix+1) - 1 ' Calculate length for output array and ...
  ReDim outputText(sizeOfOutput)                        ' resize the latter accordingly

  i = 0 ' counter for array element in output array
  dim l ' yet another counter ;-)
  
  For j = 0 To sizeOfInput Step 1
    For k = 0 To sizeOfSuffix Step 1
      For l = 0 To sizeOfPrefix Step 1
        If Not inputText(j) = "" Then
           outputText(i) = prefix(l) & RTrim(inputText(j)) & suffix(k)
        End If
        i = i + 1
        If i > sizeOfOutput Then Exit For
      Next
    Next
  Next

  If sel = 0 Then
    myEditor.Text(Join(outputText,vbCrLF))
  Else
    myEditor.selText(Join(outputText,vbCrLF))
  End If

End Sub

'===============================================================================
'                  Konjugieren
'===============================================================================
Private Function determineGroup(verb)
' Auxiliary function for the "Konjugieren" subroutine.
' This function divides German verbs into three different groups,
' based on a simple regular expression check.
' For the meaning of the groups, see the beginning of Sub "Konjugieren".
' It assumes that every verb in the list is in its infinitive form.
' Obviously, it has to be modified for other languages.
' For example in Latin 

  If runRegExpMatch(verb,"en\b") Then
    determineGroup = "group1"
  End If
  If runRegExpMatch(verb,"[dt]en\b") Or runRegExpMatch(verb,"[bfgkmpt]nen\b") Or runRegExpMatch(verb,"chnen\b") Then
    determineGroup = "group2"
  End If
  If runRegExpMatch(verb,"[lr]n\b") Then
    determineGroup = "group3"
  End If
End Function

Private Function isIrregular(verb)
' Auxiliary function for the "Konjugieren" subroutine.
' This function can of course NOT magically determine whether a verb is 
' irregular or not.
' It just checks for the '#' char at the beginning of the line, which I used
' as a marker.

  If runRegExpMatch(verb,"^\#(.)+") Then
    isIrregular = True
  Else
    isIrregular = False
  End If
End Function

Sub Konjugieren
  Dim suffix_1(5) ' An array of strings containing the suffixes for group1.)
  suffix_1(0) = "en" ' (See function determineGroup.)
  suffix_1(1) = "e"
  suffix_1(2) = "st"
  suffix_1(3) = "te"
  suffix_1(4) = "ten"
  suffix_1(5) = "test"

  Dim suffix_2(5) ' An array of strings containing the suffixes for group2
  suffix_2(0) = "en"
  suffix_2(1) = "e"
  suffix_2(2) = "est"
  suffix_2(3) = "ete"
  suffix_2(4) = "eten"
  suffix_2(5) = "etest"

  Dim suffix_3(5) ' An array of strings containing the suffixes for group3
  suffix_3(0) = "n"
  suffix_3(1) = "e"
  suffix_3(2) = "st"
  suffix_3(3) = "te"
  suffix_3(4) = "ten"
  suffix_3(5) = "test"

  ' Create new editor object and assign the active window
  If editorsCount = 0 Then Exit Sub
  set myEditor = newEditor()
  myEditor.assignActiveEditor

  ' Get selected text, if any.
  If myEditor.selText="" Then  ' If no text is selected, then ...
    If MsgBox(confText, 36, confTitle) = vbNo Then Exit Sub 'ask if user wants to apply it to the whole document
    inputText = split(myEditor.Text,vbCrLf,-1) ' get all text from active window, parse into array
    sel = 0
  Else
    inputText = split(myEditor.selText,vbCrLf,-1) ' Get selected text, parse into array
    sel = 1
  End If

  sizeOfInput = UBound(inputText,1)  ' Get length of input array (number of lines)
  sizeOfOutput = 0 ' Initialization, this will be resized later on.
  ReDim outputText(1)
  Dim suffix() ' an array that will store the suffices actually used, based on the grouping below
  dim tmp ' stores a string from input for manipulation
  dim l   ' counter for suffix array

  dim counter0 ' statistics counters
  dim counter1
  dim counter2
  dim counter3
  counter0 = 0
  counter1 = 0
  counter2 = 0
  counter3 = 0

  i = 0 ' counter for array element in output array
  For j = 0 To sizeOfInput Step 1
    If Not inputText(j) = "" Then
       outputText(i) = RTrim(inputText(j))  ' RTrim deletes trailing spaces

       ' The following If-ElseIf test checks whether a word belongs to
       ' group 1-3, or neither.

       If determineGroup(outputText(i)) = "group1" Then
         ' MsgBox "Group 1!" ' <- For testing purposes only
         counter1 = counter1 + 1 ' statistics counter
         tmp = Left(outputText(i),Len(outputText(i))-2)
         ' The above line cuts off the last two letters.
         If isIrregular(outputText(i)) Then 
           sizeOfSuffix = UBound(suffix_1,1) - 3
           ' sizeOfSuffix must be number of endings common to regular and
           ' irregular forms in this case
         Else
           sizeOfSuffix = UBound(suffix_1,1)
         End If
         ReDim suffix(sizeOfSuffix)
         For l = 0 to sizeOfSuffix Step 1
           suffix(l) = suffix_1(l)
         Next
         sizeOfOutput = sizeOfOutput + (sizeOfSuffix+1) ' Calculate length for output array and ...
         ReDim Preserve outputText(sizeOfOutput)        ' resize the latter accordingly

       ElseIf determineGroup(outputText(i)) = "group2" Then
         counter2 = counter2 + 1 ' statistics
         tmp = Left(outputText(i),Len(outputText(i))-2)
         If isIrregular(outputText(i)) Then 
           sizeOfSuffix = UBound(suffix_2,1) - 3
           ' sizeOfSuffix must be number of endings common to regular and
           ' irregular forms
         Else
           sizeOfSuffix = UBound(suffix_2,1)
         End If
         ReDim suffix(sizeOfSuffix)
         For l = 0 to sizeOfSuffix Step 1
           suffix(l) = suffix_2(l)
         Next
         sizeOfOutput = sizeOfOutput + (sizeOfSuffix+1) ' Calculate length for output array and ...
         ReDim Preserve outputText(sizeOfOutput)        ' resize the latter accordingly

       ElseIf determineGroup(outputText(i)) = "group3" Then
         counter3 = counter3 + 1 ' statistics
         tmp = Left(outputText(i),Len(outputText(i))-1)
         If isIrregular(outputText(i)) Then 
           sizeOfSuffix = UBound(suffix_3,1) - 3
           ' sizeOfSuffix must be number of endings common to regular and
           ' irregular forms
         Else
           sizeOfSuffix = UBound(suffix_3,1)
         End If
         ReDim suffix(sizeOfSuffix)
         For l = 0 to sizeOfSuffix Step 1
           suffix(l) = suffix_3(l)
         Next
         sizeOfOutput = sizeOfOutput + (sizeOfSuffix+1) ' Calculate length for output array and ...
         ReDim Preserve outputText(sizeOfOutput)        ' resize the latter accordingly

       Else
         sizeOfSuffix = 0
         ReDim suffix(sizeOfSuffix)
         tmp = outputText(i)
         suffix(0) = ""
         counter0 = counter0 + 1
         sizeOfOutput = sizeOfOutput + 1
         ReDim Preserve outputText(sizeOfOutput)

       End If

       For k = 0 To sizeOfSuffix Step 1
         outputText(i) = tmp & suffix(k)
         i = i + 1
         If i > sizeOfOutput Then Exit For
      Next
    End If
  Next

  If openInNew = False Then
    If sel = 0 Then
      myEditor.Text(Join(outputText,vbCrLF))
    Else
      myEditor.selText(Join(outputText,vbCrLF))
    End If
  Else ' (open in new editor window)
    dim ed2
    set ed2 = newEditor()
    ed2.NewFile()          
    ed2.Text(Join(outputText,vbCrLF)) ' replaces all text in the new window
  End If

  ' Optional statistics. Set showStats to False to avoid the pop-up.
  If showStats = True Then MsgBox _
      counter1 & Chr(9) &"words in group 1," & vbCrLf _
    & counter2 & Chr(9) &"words in group 2," & vbCrLf _
    & counter3 & Chr(9) &"words in group 3," & vbCrLf _
    & counter0 & Chr(9) &"words not in any group.",vbOkOnly,"Summary"

End Sub

'===============================================================================
'                  Regular Expression Search
'===============================================================================
Private Function runRegExpMatch(haystack, needle)
    Dim regEx
    Set regEx = New RegExp
    regEx.Pattern    = needle
    regEx.Global     = TRUE
    regEx.IgnoreCase = TRUE
    runRegExpMatch = regEx.Test(haystack)
End Function
